/*
 ============================================================================
 Name        : EtaPhotReggeModel.c
 Author      : Jannes Nys
 Version     : v1.0 November 2016
 Copyright   : 
 Publication : unpublished + eta-MAID
 Description : Photoproduction of a pseudoscalar : gamma + N --> 0- + N'
               Applied to gamma + p --> eta + p
               To change the model, change 'CGLN_Ai'
               The model is based on Regge exchanges  + the MAID solution
 ============================================================================
*/
 
#include<EtaPhotReggeModelAmplitudes.h>

// *********************************************************************************
// this is the model, defined in terms of the Ai
void Regge_Ai(struct Kin kin, double complex CGLNA[5])
{
        double s = kin.s;
        double t = kin.t;
	/*
	 * Model for gamma p --> eta p (J. Nys and V. Mathieu, unpublished)
         * Jannes Nys November 2016
	 */

	// Model parameters
	// ================
	// Regge trajectories
        double alphRho[2] = { 1.0-0.9*MRHO*MRHO , 0.9 };
        double alphOmega[2] = { 1.0-0.9*MRHO*MRHO , 0.9 };
        double alphB[2] = { -0.7*MPI0*MPI0 , 0.7 };
        double alphH[2] = { -0.7*MPI0*MPI0 , 0.7 }; // degenerate h-b slope
	
	// coupling constants
        double g1rho = 3.43352;		// s-channel nucleon helicity flip
        double g4rho = -1.39707;			// s-channel nucleon helicity non-flip
        double g1omega = 0.115605;			// s-channel nucleon helicity flip
        double g4omega = -3.34608;		// s-channel nucleon helicity non-flip
        double g2b = 4.94633;
        double r1 = 3.00061;
        double r4 = 1.97379;
        double r2 = 6.20409;
        double g2h = g2b/1.5;
	
        double complex Rrho, Romega, Rb, Rh;
        double complex aRho, aOmega, aB, aH;
	
	// calculate the Regge trajectories for the given kinematics
        aRho = alphRho[0] + t*alphRho[1] ;
        aOmega = alphOmega[0] + t*alphOmega[1];
        aB = alphB[0] + t*alphB[1] ;
        aH = alphH[0] + t*alphH[1];
	
        double complex u = kin.u;
        double complex nu = kin.nu;

	// Regge propagators and signature factors
	// 			normalization	sin and gamma		signature							Regge behaviour
        Rrho = 		-alphRho[1] *	realgamma(-aRho,0) *	( cexp(-I*M_PI*aRho)-1.0 )/2.0 * 		cpow(nu,aRho-1.0);
        Romega = 	-alphOmega[1] *	realgamma(-aOmega,0) *	( cexp(-I*M_PI*aOmega)-1.0 )/2.0 * 	cpow(nu,aOmega-1.0);
        Rb = 		-alphB[1] *		realgamma(-aB,0) *		( cexp(-I*M_PI*aB)-1.0 )/2.0 * 		cpow(nu,aB-1.0);
        Rh = 		-alphH[1] *		realgamma(-aH,0) *		( cexp(-I*M_PI*aH)-1.0 )/2.0 *		cpow(nu,aH-1.0);
	// where -pi/sin(pi*alpha)/Gamma(alpha+1) = Gamma(-alpha)
	
	// Scalar amplitudes:
	// NOTE: here we work with the A_2, while we work with A_2^\prime in the paper
	CGLNA[1] = 0.0;	CGLNA[2] = 0.0;	CGLNA[3] = 0.0;	CGLNA[4] = 0.0;
	
        if(t != 0.){

            // ISOSCALAR
            // Omega contribution
            CGLNA[1] += g1omega*t*Romega*pow(r1,aOmega-1);
            CGLNA[2] += -g1omega*Romega*pow(r1,aOmega-1);
            CGLNA[4] += g4omega*aOmega*Romega*pow(r4,aOmega-1);
            // h contribution
            CGLNA[2] += g2h*Rh*pow(r2,aH-1);

            //// ISOVECTOR
            // Rho contribution
            CGLNA[1] += g1rho*t*Rrho*pow(r1,aRho-1);
            CGLNA[2] += -g1rho*Rrho*pow(r1,aRho-1);
            CGLNA[4] += g4rho*aRho*Rrho*pow(r4,aRho-1);
            // b contribution
            CGLNA[2] += g2b*Rb*pow(r2,aB-1);
        }

	//
        return;
}

// *********************************************************************************
// rotate from Ai to Fi
void CGLNA2F(struct Kin var, double complex Ai[5], double complex Fi[5]){
	/*
	 * Compute CGLN Fi(s,t) from CGLN Ai(s,t)
	 * Ai are in GeV^-{2,4,3,3}
	 * Fi are in mFm
	 */
        double complex w;
        double complex E2p, E2m, E4p, E4m;

	w = csqrt(var.s);
	E2p = var.E2s + var.m2 ;
	E2m = var.E2s - var.m2 ;
	E4p = var.E4s + var.m4 ;
	E4m = var.E4s - var.m4 ;

	Fi[1] = (w - var.m2) * Ai[1] + (var.m3*var.m3 - var.t)/2 * (Ai[3] - Ai[4]);
	Fi[1] = Fi[1]  + ( w - var.m2 ) * ( w - var.m4 ) * Ai[4];
	Fi[1] = Fi[1] * csqrt( E2p * E4p ) / ( 8 * M_PI * w);

	Fi[2] = -(w + var.m2) * Ai[1] + (var.m3*var.m3 - var.t)/2 * (Ai[3] - Ai[4]);
	Fi[2] = Fi[2]  + ( w + var.m2 ) * ( w + var.m4 ) * Ai[4];
	Fi[2] = Fi[2] * csqrt( E2m * E4m ) / ( 8 * M_PI * w);

	Fi[3] = (w + var.m2) * (  (w - var.m2)*Ai[2] + Ai[3] - Ai[4] );
	Fi[3] = Fi[3] * csqrt( E2m * E4p) * var.qs  / ( 8 * M_PI * w);

	Fi[4] = (w - var.m2) * ( -(w + var.m2)*Ai[2] + Ai[3] - Ai[4] );
	Fi[4] = Fi[4] * csqrt( E2p * E4m) * var.qs  / ( 8 * M_PI * w);

	// above Fi[] are in GeV^-1
	// conversion in mFm
	// HBC = 0.19732 fm.GeV ; 1b = 100 fm^2 ; HBC^2 = (0.19732)^2 (10 mb).GeV^2
	// 10^4 HBC^2 = 389.35 mub.GeV^2
	double hbarc = 197.32; // 1000*hbar*c
	Fi[1] = hbarc*Fi[1];
	Fi[2] = hbarc*Fi[2];
	Fi[3] = hbarc*Fi[3];
	Fi[4] = hbarc*Fi[4];

	return;
}

// *********************************************************************************
double complex Eg2W(double Eg)
{
    // lab to s-channel cm conversion
    return csqrt( MP*MP + 2.0*MP*sqrt(Eg*Eg) );
}


// *********************************************************************************
void kinematics(double complex s, double complex t, double mass[5], struct Kin *var)
{
	double m12, m22, m32, m42; 	// masses squared
        double complex t0, t1, u ;

	var->s = s;
	var->t = t;

	var->m1 = mass[1];
	var->m2 = mass[2];
	var->m3 = mass[3];
	var->m4 = mass[4];

	m12 = mass[1] * mass[1] ;
	m22 = mass[2] * mass[2] ;
	m32 = mass[3] * mass[3] ;
	m42 = mass[4] * mass[4] ;

	var->ks = csqrt( lambda(s, m12, m22) / 4 / s );
	var->qs = csqrt( lambda(s, m32, m42) / 4 / s );
	var->kt = csqrt( lambda(t, m12, m32) / 4 / t );
	var->pt = csqrt( lambda(t, m22, m42) / 4 / t );

	// nuclei energies in s- and t-channel frames
	var->E2s = ( s + m22 - m12 ) / 2 / csqrt(s);
	var->E4s = ( s + m42 - m32 ) / 2 / csqrt(s);
	var->E2t = ( t + m22 - m42 ) / 2 / csqrt(t);
	var->E4t = ( t + m42 - m22 ) / 2 / csqrt(t);

	// t1 = tmin ; t0 = tmax ; t0 < t < t1
	t1 = cpow(m12 - m32 - m22 + m42,2)/(4*s) - (var->ks - var->qs) * (var->ks - var->qs);
	t0 = t1 - 4 * var->ks *var->qs ;
	u = - s - t + m12 + m22 + m32 + m42 ;	// Mandelstam s variable
        var->u = u;
	var->phi = s * (t - t1) * (t0 - t) ;	// Kibble function
        var->nu = (s-u)/(4.0*var->m2);

	var->coss = 1 + (t - t1)/(2 * var->qs * var->ks) ;
	//var->cost = (t*(s-u) + (m12-m32) * (m22-m42) )/(4 * t * var->pt * var->kt) ;
	var->cost = (t*(s-u) + (m12-m32) * (m22-m42) )/(csqrt(lambda(var->t,m12,m32) * lambda(var->t,m22,m42) ) ) ;
	var->sins = csqrt( var->phi / s ) / ( 2 * var->qs * var->ks) ;
	var->sint = csqrt( var->phi / t ) / ( 2 * var->kt * var->pt) ;
	var->cossh = csqrt( (1 + var->coss ) / 2 );
	var->sinsh = csqrt( (1 - var->coss ) / 2 );
	var->costh = csqrt( (1 + var->cost ) / 2 );
	var->sinth = csqrt( (1 - var->cost ) / 2 );

	return ;
}

// *********************************************************************************
double Cos2T(double W, double z, double mass[5]){
	/*
         * Compute t from W and cos = z
	 */
	double t;
        double s = W*W;
	double ma, mb, mc, md;			// masses of the particles
	ma = mass[1]; mb = mass[2]; mc = mass[3]; md = mass[4];

	double qi, qf;
	qi = sqrt(lambda(s, ma*ma, mb*mb) /4/s);
	qf = sqrt(lambda(s, mc*mc, md*md) /4/s);
	double som = ma*ma + mb*mb + mc*mc + md*md;

	t = 2*qi*qf*z + som/2 - s/2 - (ma*ma - mb*mb) * (mc*mc - md*md) /2/s ;
	return t;
}

// *********************************************************************************
double T2Cos(double W, double t, double mass[5]){
	/*
         * Compute z = cos from W
	 */
	double z;
        double s = W*W;
	double ma, mb, mc, md;			// masses of the particles
	ma = mass[1]; mb = mass[2]; mc = mass[3]; md = mass[4];

	double qi, qf;
	qi = sqrt(lambda(s, ma*ma, mb*mb) /4/s);
	qf = sqrt(lambda(s, mc*mc, md*md) /4/s);
	double som = ma*ma + mb*mb + mc*mc + md*md;

	z = (t + (ma*ma - mb*mb)*(mc*mc - md*md)/(2.0*s) - som/2.0 + s/2.0 ) /(2*qi*qf);
	return z;
}


// *********************************************************************************
double complex lambda(double complex a, double b, double c){
// triangle function
	return a*a + b*b + c*c - 2*(a*b + b*c + c*a);
}

//// *********************************************************************************
double complex realgamma(double complex z, int OPT)
// Calculate Gamma function for real z
{
	if(OPT == 0){
		return tgamma(creal(z));
	}else{
		return clog(tgamma(creal(z))); // ln gamma
	}
}
		
// *********************************************************************************
double cnorm(double complex x)
{
    return creal(x)*creal(x) + cimag(x)*cimag(x);
}
