/*
 ============================================================================
 Name        : EtaPhotReggeModel.h
 Author      : Jannes Nys
 Version     : v1.0 November 2016
 Copyright   : 
 Publication : unpublished + eta-MAID
 Description : Photoproduction of a pseudoscalar : gamma + N --> 0- + N'
               Applied to gamma + p --> eta + p
               To change the model, change 'CGLN_Ai'
               The model is based on Regge exchanges  + the MAID solution
 ============================================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <complex.h>    	/* Standard Library of Complex Numbers */

#ifndef ETAPHOTREGGEMODEL_H
#define ETAPHOTREGGEMODEL_H

// UNITS = GeV
#define MP  0.938272046
#define META 0.547853
#define MRHO 0.775
#define MOMEGA 0.782
#define MB1 1.229 
#define MH1 1.170
#define MPI0 0.1349766
#define EPS 0.0000
#define HBC 0.19732;
#define M_PI 3.14159265358979323846264338327950288

/*
 * ONLY VALID INSIDE THE PHYSICAL S-CHANNEL REGION!!! -1 <= t <= tmin
 *  only photoproduction Q^2 = 0 = M1^2
 */


// Structure with all the kinematical quantities
struct Kin
{
	double m1;				// Mass particle 1
	double m2;				// Mass particle 2
	double m3;				// Mass particle 3
	double m4;				// Mass particle 4
	double complex s;				// Mandelstam s variable
        double complex t;				// Mandelstam t variable
        double complex phi;				// phi = stu + ...
        double complex ks;		// photon  momentum in s-channel frame
        double complex qs;		// eta    momentum in s-channel frame
        double complex kt;		// photon  momentum in t-channel frame
        double complex pt;		// nucleon momentum in t-channel frame
        double complex E2s;		// initial nucleon energy in s-channel frame
        double complex E4s;		// final   nucleon energy in s-channel frame
        double complex E2t;		// initial nucleon energy in t-channel frame
        double complex E4t;		// final   nucleon energy in t-channel frame
        double complex coss;	// cos \theta_s
        double complex sins;	// sin \theta_s
        double complex cossh;	// cos \theta_s/2
        double complex sinsh;	// sin \theta_s/2
        double complex cost;	// cos \theta_t
        double complex sint;	// sin \theta_t
        double complex costh;	// cos \theta_t/2
        double complex sinth;	// sin \theta_t/2
        double complex u;       // mandelstam s variable
        double complex nu;      // crossing variable (s-u)/(4 M_N)
};

/********* HEADER ********* */
/*
 ============================================================================
 Description of the subroutines:
    Regge_Ai
 	 CGLN Scalar amplitudes Ai - the model has to be specified there
 CGLNA2F
 	 transformation between CGLN Fi (ouputs) and Ai (inputs)
 kinematics
 	 compute all kinematical quantities (s- and t-channel)
 	 and store then in a struct Kin

 functions:
 ---------
 Cos2T
 	 return Mandelstam t from cos(theta_s)
 T2Cos
 	 return cos(theta_s) from Mandelstam t
 realgamma
 	 return gamma(x) or log( gamma(x) )
 lambda
 	 return a*a + b*b + c*c - 2*(a*b + b*c + c*a)
 ============================================================================
 */

double cnorm(double complex x);
double complex realgamma(double complex z, int OPT);
void Regge_Ai(struct Kin kin, double complex CGLNA[5]);
double Cos2T(double W, double z, double mass[5]);
double T2Cos(double W, double t, double mass[5]);
void CGLNA2F(struct Kin var, double complex Ai[5], double complex Fi[5]);
void kinematics(double complex s, double complex t, double mass[], struct Kin *var);
double complex Eg2W(double Eg);
double complex lambda(double complex a, double b, double c);


#endif
